import 'package:flutter/material.dart';

class Timeline extends StatelessWidget {
  const Timeline({
    @required this.children,
    this.indicators,
    this.isLeftAligned = true,
    this.itemGap = 12.0,
    this.gutterSpacing = 4.0,
    this.padding = const EdgeInsets.all(8),
    this.controller,
    this.lineColor = Colors.grey,
    this.physics,
    this.shrinkWrap = true,
    this.primary = false,
    this.reverse = false,
    this.indicatorSize = 30.0,
    this.lineGap = 4.0,
    this.indicatorColor = Colors.blue,
    this.indicatorStyle = PaintingStyle.fill,
    this.strokeCap = StrokeCap.butt,
    this.strokeWidth = 2.0,
    this.style = PaintingStyle.stroke,
    this.detailLength,
  })  : itemCount = children.length,
        assert(itemGap >= 0),
        assert(lineGap >= 0),
        assert(indicators == null || children.length == indicators.length);

  final List<Widget> children;
  final double itemGap;
  final double gutterSpacing;
  final List<Widget> indicators;
  final bool isLeftAligned;
  final EdgeInsets padding;
  final ScrollController controller;
  final int itemCount;
  final ScrollPhysics physics;
  final bool shrinkWrap;
  final bool primary;
  final bool reverse;

  final Color lineColor;
  final double lineGap;
  final double indicatorSize;
  final Color indicatorColor;
  final PaintingStyle indicatorStyle;
  final StrokeCap strokeCap;
  final double strokeWidth;
  final PaintingStyle style;
  final List<int> detailLength;

  @override
  Widget build(BuildContext context) {
    Size deviceSize = MediaQuery.of(context).size;
    bool isPortrait = MediaQuery.of(context).orientation == Orientation.portrait;

    return Container(
      child: Column(
        children: children
            .asMap()
            .map((index, item) {
              final child = children[index];

              Widget indicator;
              if (indicators != null) {
                indicator = indicators[index];
              }

              double lineHeight = (deviceSize.width / (isPortrait ? 2.6 : 4)) - 20;
              if ((detailLength[index] + 50) > lineHeight) {
                lineHeight = detailLength[index] + 80.0;
              } else if (detailLength[index] > 40) {
                lineHeight = lineHeight + 45.0;
              }

              final isFirst = index == 0;
              final isLast = index == itemCount - 1;
              final timelineTile = <Widget>[
                CustomPaint(
                  foregroundPainter: _TimelinePainter(
                    hideDefaultIndicator: indicator != null,
                    lineColor: lineColor,
                    indicatorColor: indicatorColor,
                    indicatorSize: indicatorSize,
                    indicatorStyle: indicatorStyle,
                    isFirst: isFirst,
                    isLast: isLast,
                    lineGap: lineGap,
                    strokeCap: strokeCap,
                    strokeWidth: strokeWidth,
                    style: style,
                    itemGap: itemGap,
                  ),
                  child: SizedBox(
                    height: lineHeight, //double.maxFinite,
                    width: indicatorSize,
                    child: indicator,
                  ),
                ),
                SizedBox(width: gutterSpacing),
                Expanded(child: child),
              ];
              return MapEntry(
                index,
                Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: isLeftAligned ? timelineTile : timelineTile.reversed.toList(),
                ),
              );
            })
            .values
            .toList(),
      ),
    );
    // return ListView.separated(
    //   padding: padding,
    //   separatorBuilder: (_, __) => SizedBox(height: itemGap),
    //   physics: physics,
    //   shrinkWrap: shrinkWrap,
    //   itemCount: itemCount,
    //   controller: controller,
    //   reverse: reverse,
    //   primary: primary,
    //   itemBuilder: (context, index) {
    //     final child = children[index];

    //     Widget indicator;
    //     if (indicators != null) {
    //       indicator = indicators[index];
    //     }

    //     final isFirst = index == 0;
    //     final isLast = index == itemCount - 1;

    //     final timelineTile = <Widget>[
    //       CustomPaint(
    //         foregroundPainter: _TimelinePainter(
    //           hideDefaultIndicator: indicator != null,
    //           lineColor: lineColor,
    //           indicatorColor: indicatorColor,
    //           indicatorSize: indicatorSize,
    //           indicatorStyle: indicatorStyle,
    //           isFirst: isFirst,
    //           isLast: isLast,
    //           lineGap: lineGap,
    //           strokeCap: strokeCap,
    //           strokeWidth: strokeWidth,
    //           style: style,
    //           itemGap: itemGap,
    //         ),
    //         child: SizedBox(
    //           height: double.infinity,
    //           width: indicatorSize,
    //           child: indicator,
    //         ),
    //       ),
    //       SizedBox(width: gutterSpacing),
    //       Expanded(child: child),
    //     ];

    //     return Row(
    //       mainAxisAlignment: MainAxisAlignment.start,
    //       children: isLeftAligned ? timelineTile : timelineTile.reversed.toList(),
    //     );
    //   },
    // );
  }
}

class _TimelinePainter extends CustomPainter {
  _TimelinePainter({
    @required this.hideDefaultIndicator,
    @required this.indicatorColor,
    @required this.indicatorStyle,
    @required this.indicatorSize,
    @required this.lineGap,
    @required this.strokeCap,
    @required this.strokeWidth,
    @required this.style,
    @required this.lineColor,
    @required this.isFirst,
    @required this.isLast,
    @required this.itemGap,
  })  : linePaint = Paint()
          ..color = lineColor
          ..strokeCap = strokeCap
          ..strokeWidth = strokeWidth
          ..style = style,
        circlePaint = Paint()
          ..color = indicatorColor
          ..style = indicatorStyle;

  final bool hideDefaultIndicator;
  final Color indicatorColor;
  final PaintingStyle indicatorStyle;
  final double indicatorSize;
  final double lineGap;
  final StrokeCap strokeCap;
  final double strokeWidth;
  final PaintingStyle style;
  final Color lineColor;
  final Paint linePaint;
  final Paint circlePaint;
  final bool isFirst;
  final bool isLast;
  final double itemGap;

  @override
  void paint(Canvas canvas, Size size) {
    final indicatorRadius = indicatorSize / 2;
    final halfItemGap = itemGap / 2;
    final indicatorMargin = indicatorRadius + lineGap;

    final top = size.topLeft(Offset(indicatorRadius, 0.0 - halfItemGap));
    final centerTop = size.centerLeft(
      Offset(indicatorRadius, -indicatorMargin),
    );

    final bottom = size.bottomLeft(Offset(indicatorRadius, 0.0 + halfItemGap));
    final centerBottom = size.centerLeft(
      Offset(indicatorRadius, indicatorMargin),
    );

    if (!isFirst) canvas.drawLine(top, centerTop, linePaint);
    if (!isLast) canvas.drawLine(centerBottom, bottom, linePaint);

    if (!hideDefaultIndicator) {
      final Offset offsetCenter = size.centerLeft(Offset(indicatorRadius, 0));

      canvas.drawCircle(offsetCenter, indicatorRadius, circlePaint);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}
